# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: hello.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0bhello.proto\"3\n\x0cHelloRequest\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\t\x12\x15\n\x05skill\x18\x02 \x01(\x0b\x32\x06.Skill\"\x84\x01\n\rHelloResponse\x12\x0e\n\x06result\x18\x01 \x01(\t\x12\x31\n\nmap_result\x18\x02 \x03(\x0b\x32\x1d.HelloResponse.MapResultEntry\x1a\x30\n\x0eMapResultEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\x05:\x02\x38\x01\"\x15\n\x05Skill\x12\x0c\n\x04name\x18\x01 \x01(\t27\n\x0bGrpcService\x12(\n\x05hello\x12\r.HelloRequest\x1a\x0e.HelloResponse\"\x00\x42\x03\x80\x01\x01\x62\x06proto3')



_HELLOREQUEST = DESCRIPTOR.message_types_by_name['HelloRequest']
_HELLORESPONSE = DESCRIPTOR.message_types_by_name['HelloResponse']
_HELLORESPONSE_MAPRESULTENTRY = _HELLORESPONSE.nested_types_by_name['MapResultEntry']
_SKILL = DESCRIPTOR.message_types_by_name['Skill']
HelloRequest = _reflection.GeneratedProtocolMessageType('HelloRequest', (_message.Message,), {
  'DESCRIPTOR' : _HELLOREQUEST,
  '__module__' : 'hello_pb2'
  # @@protoc_insertion_point(class_scope:HelloRequest)
  })
_sym_db.RegisterMessage(HelloRequest)

HelloResponse = _reflection.GeneratedProtocolMessageType('HelloResponse', (_message.Message,), {

  'MapResultEntry' : _reflection.GeneratedProtocolMessageType('MapResultEntry', (_message.Message,), {
    'DESCRIPTOR' : _HELLORESPONSE_MAPRESULTENTRY,
    '__module__' : 'hello_pb2'
    # @@protoc_insertion_point(class_scope:HelloResponse.MapResultEntry)
    })
  ,
  'DESCRIPTOR' : _HELLORESPONSE,
  '__module__' : 'hello_pb2'
  # @@protoc_insertion_point(class_scope:HelloResponse)
  })
_sym_db.RegisterMessage(HelloResponse)
_sym_db.RegisterMessage(HelloResponse.MapResultEntry)

Skill = _reflection.GeneratedProtocolMessageType('Skill', (_message.Message,), {
  'DESCRIPTOR' : _SKILL,
  '__module__' : 'hello_pb2'
  # @@protoc_insertion_point(class_scope:Skill)
  })
_sym_db.RegisterMessage(Skill)

_GRPCSERVICE = DESCRIPTOR.services_by_name['GrpcService']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\200\001\001'
  _HELLORESPONSE_MAPRESULTENTRY._options = None
  _HELLORESPONSE_MAPRESULTENTRY._serialized_options = b'8\001'
  _HELLOREQUEST._serialized_start=15
  _HELLOREQUEST._serialized_end=66
  _HELLORESPONSE._serialized_start=69
  _HELLORESPONSE._serialized_end=201
  _HELLORESPONSE_MAPRESULTENTRY._serialized_start=153
  _HELLORESPONSE_MAPRESULTENTRY._serialized_end=201
  _SKILL._serialized_start=203
  _SKILL._serialized_end=224
  _GRPCSERVICE._serialized_start=226
  _GRPCSERVICE._serialized_end=281
# @@protoc_insertion_point(module_scope)
